#version 140              // delete this line if using GLSL 1.2

//Light uniforms
uniform vec4 ambient_color;
uniform vec4 light_color;
uniform vec4 spec_color;
uniform vec4 light_position;
        
//Attenuation
// uniform float k0;
// uniform float k1;
// uniform float k2;

//Meterials
uniform vec4 ka;
uniform vec4 kd;
uniform vec4 ks;
/*uniform float shininess;*/

//Textures
uniform sampler2D ctexture;

//Camera position
uniform vec4 camPos;

//Shadow maps
uniform sampler2D sm_0;
uniform sampler2D sm_1;
uniform sampler2D sm_2;
uniform sampler2D sm_3;
uniform sampler2D sm_4;
uniform sampler2D sm_5;


//Fragment-shader input variables
in vec4 frag_color;
in vec3 world_normal;
in vec4 world_position;
in vec2 fragTexCoord;

//Shadowmap coords
in vec4 SM_tex_coord_0;
in vec4 SM_tex_coord_1;
in vec4 SM_tex_coord_2;
in vec4 SM_tex_coord_3;
in vec4 SM_tex_coord_4;
in vec4 SM_tex_coord_5;

// Fragment-shader output variable (-> stored in the frame-buffer, i.e. "the pixel you see")
out vec4 fragColor;

void main()
{

	//Constants for now
	float shininess = 1.0;
	float k0 = 1.0;
	float k1 = 0.00008;
	float k2 = 0.00004;

	// Vector from surface to light position
	vec3 normal = normalize(world_normal);
    vec3 position = world_position.xyz / world_position.w;
    
    //Light direction
    vec3 aux       = light_position.xyz - position;
    float dist     = length(aux);
    vec3 light_dir = normalize(aux);
   
    //Angle
    float NdotL = dot(normal, light_dir); 

	//Attenuation
    float attenuation = 1.0/(k0 + (k1*dist) + (k2*dist*dist));
    //float attenuation = 1.0;

	//View
	vec3 view = normalize(camPos.xyz - position);
	
	//Halfway
    vec3 H = normalize(light_dir + view);
    
    //Texture
    vec4 texel = texture(ctexture, fragTexCoord);
    
    //Ambient
    vec4 ambient = ka * ambient_color;
    
    //Diffuse
    vec4 diffuse = kd * light_color * max(0.0, NdotL);
    
    //Angle
	//float NdotH = max(0.0, dot(normal, H));
	//Shininess
	//float sf = pow(NdotH, shininess);
    
    //Specular
    //vec4 specular = ks * spec_color * sf;
 
 

	vec3 sm_l0 = SM_tex_coord_0.xyz / SM_tex_coord_0.w;
	vec3 sm_l1 = SM_tex_coord_1.xyz / SM_tex_coord_1.w;
	vec3 sm_l2 = SM_tex_coord_2.xyz / SM_tex_coord_2.w;
	vec3 sm_l3 = SM_tex_coord_3.xyz / SM_tex_coord_3.w;
	vec3 sm_l4 = SM_tex_coord_4.xyz / SM_tex_coord_4.w;
	vec3 sm_l5 = SM_tex_coord_5.xyz / SM_tex_coord_5.w;
	
	float depth_0 = texture2D(sm_0, sm_l0.xy).x;
	float depth_1 = texture2D(sm_1, sm_l1.xy).x;
	float depth_2 = texture2D(sm_2, sm_l2.xy).x;
	float depth_3 = texture2D(sm_3, sm_l3.xy).x;
	float depth_4 = texture2D(sm_4, sm_l4.xy).x;
	float depth_5 = texture2D(sm_5, sm_l5.xy).x;

	
	//Shadow factor: 1.0 = light, 0.0 = shadow
    float factor = 0.0;

	if ((depth_0 >= sm_l0.z) ||
		(depth_1 >= sm_l1.z) ||
		(depth_2 >= sm_l2.z) ||
		(depth_3 >= sm_l3.z) ||
		(depth_4 >= sm_l4.z) ||
		(depth_5 >= sm_l5.z)) {
		//In light!
		factor = 1.0;
	}
	
	vec4 color_out = attenuation * texel * (ambient + diffuse*factor);
	
	color_out.w = 1.0f;
	
	fragColor = color_out;
}